<?php

/**
 * @file
 * Connector class for Lionbridge onDemand API service.
 */

/**
 * Connects Lionbridge API to TMGMT.
 */
class TMGMTLionbridgeConnector {

  /**
   * API Key ID.
   *
   * @var string
   */
  protected $keyId;

  /**
   * API Secret ID.
   *
   * @var string
   */
  protected $secretId;

  /**
   * API url.
   *
   * @var string
   */
  protected $endPoint;

  /**
   * Active currency.
   *
   * @var string
   */
  protected $currency;

  /**
   * Api version - Check current version here http://api-docs.liondemand.com/.
   *
   * @var string
   */
  protected $apiVersion = '2015-02-23';

  /**
   * List of allowed currencies.
   *
   * @var array
   */
  protected $allowedCurrencies = ['USD', 'EUR', 'GBP'];

  /**
   * Optional Purchase Order Number.
   *
   * @var string
   */
  protected $poNumber;

  /**
   * Construct class and set default params.
   *
   * @param \TMGMTTranslator $translator
   *   Constructs the connector.
   */
  public function __construct(TMGMTTranslator $translator) {
    $this->keyId = $translator->getSetting('access_key_id');
    $this->secretId = $translator->getSetting('access_key');
    $this->currency = $translator->getSetting('currency');
    $this->endPoint = $translator->getSetting('endpoint');
    $this->poNumber = $translator->getSetting('po_number');
  }

  /**
   * Return xml with account information.
   *
   * @return string
   *   Gets Lionbridge account info.
   */
  public function accountInformation() {
    return $this->getResource('/api/account/info');
  }

  /**
   * Gets the currency property.
   *
   * @return string
   *   The currency (USD, EUR, GBP).
   */
  public function getCurrency() {
    return $this->currency;
  }

  /**
   * Gets the PO number.
   *
   * @return int
   *   The Lionbridge PO number.
   */
  public function getPoNumber() {
    return $this->poNumber;
  }

  /**
   * Gets the endpoint.
   *
   * @return string
   *   The Lionbridge API endpoint.
   */
  public function getEndPoint() {
    return $this->endPoint;
  }

  /**
   * Gets the API key ID.
   *
   * @return string
   *   The API key ID.
   */
  public function getKeyId() {
    return $this->keyId;
  }

  /**
   * Gets the API key secret ID.
   *
   * @return string
   *   The API key secret ID.
   */
  public function getSecretId() {
    return $this->secretId;
  }

  /**
   * Gets the API version.
   *
   * @return string
   *   The API version.
   */
  public function getApiVersion() {
    return $this->apiVersion;
  }

  /**
   * Gets the list of allowed currencies.
   *
   * @return array
   *   A list of currencies.
   */
  public function getAllowedCurrencies() {
    return $this->allowedCurrencies;
  }

  /**
   * Return xml with services details.
   *
   * @return object
   *   Return xml with services details.
   */
  public function listServices() {
    return $this->getResource('/api/services');
  }

  /**
   * Return xml with service details.
   *
   * @return object
   *   Return xml with service details.
   *
   * @throws TMGMTException
   */
  public function getService($service_id) {
    if (empty($service_id)) {
      throw new TMGMTException('Empty Service ID.');
    }

    return $this->getResource('/api/services/' . $service_id);
  }

  /**
   * Returns xml with lis of locales.
   *
   * @return object
   *   Returns xml with lis of locales.
   */
  public function listLocales() {
    return $this->getResource('/api/locales');
  }

  /**
   * Return xml list of all files.
   *
   * @return object
   *   Return xml list of all files.
   */
  public function listFiles() {
    return $this->getResource('/api/files');
  }

  /**
   * Return xml with details for a single project.
   *
   * @param string $project_id
   *   The project id returned from Lionbridge.
   *
   * @return object
   *   Return xml with details for a single project.
   *
   * @throws TMGMTException
   */
  public function getProject($project_id) {
    if (empty($project_id)) {
      throw new TMGMTException('Empty Project ID.');
    }

    return $this->getResource('/api/projects/' . $project_id);
  }

  /**
   * Return xml with details for a single project.
   *
   * @param string $quote_id
   *   The quote id returned from Lionbridge.
   *
   * @return object
   *   Return xml with details for a single quote.
   *
   * @throws TMGMTException
   */
  public function getQuote($quote_id) {
    if (empty($quote_id)) {
      throw new TMGMTException('Empty Quote ID.');
    }

    return $this->getResource('/api/quote/' . $quote_id);
  }

  /**
   * Authorize quote for a given job.
   *
   * @param array $quote
   *   The quote returned from getQuote().
   *
   * @return object
   *   Return xml with success or error.
   */
  public function authorizeQuote(array $quote) {
    $po_number = $this->getPoNumber();
    $projects = [];

    // Format Quote values array to pass to format_xml_elements().
    foreach ($quote['Projects'] as $project) {
      $project_files = [];
      $project_target_languages = [];

      foreach ($project['TargetLanguages'] as $target_language) {
        $project_target_languages[] = [
          'key' => 'TargetLanguage',
          'value' => $target_language,
        ];
      }

      // Process quotes with single and multiple file assets.
      if (is_numeric(implode(array_keys($project['Files']['File'])))) {
        foreach ($project['Files']['File'] as $file) {
          $file_properties = [];

          foreach ($file as $key => $value) {
            $file_properties[] = [
              'key' => $key,
              'value' => $value,
            ];
          }

          $project_files[] = [
            'key' => 'File',
            'value' => $file_properties,
          ];
        }
      }
      else {
        $project_files[] = [
          'key' => 'File',
          'value' => $project['Files']['File'],
        ];
      }

      $projects[] = [
        'key' => 'Project',
        'value' => [
          'ProjectID' => $project['ProjectID'],
          'ProjectName' => $project['ProjectName'],
          'ServiceID' => $project['ServiceID'],
          'SourceLanguage' => ['LanguageCode' => $project['SourceLanguage']['LanguageCode']],
          'TargetLanguages' => $project_target_languages,
          'Files' => $project_files,
        ],
      ];
    }

    $quote_xml = [
      'Quote' => [
        'QuoteID' => $quote['QuoteID'],
        'Status' => $quote['Status'],
        'CreationDate' => $quote['CreationDate'],
        'ServiceID' => $quote['Projects']['Project']['ServiceID'],
        'TotalTranslations' => count($quote['Projects']['Project']['Files']['File']),
        'TranslationCredit' => 0,
        'TotalCost' => $quote['TotalCost'],
        'PrepaidCredit' => $quote['PrepaidCredit'],
        'PurchaseOrderNumber' => $po_number,
        'AmountDue' => $quote['AmountDue'],
        'Currency' => $quote['Currency'],
        'Projects' => $projects,
        'SourceLanguage' => ['LanguageCode' => $quote['Projects']['Project']['SourceLanguage']['LanguageCode']],
        'TargetLanguages' => [
          [
            'key' => 'TargetLanguage',
            'value' => [
              'LanguageCode' => $quote['Projects']['Project']['TargetLanguages']['TargetLanguage']['LanguageCode'],
            ],
          ],
        ],
      ],
    ];

    $resource = $this->sendResource(
      "/api/quote/{$quote['QuoteID']}/authorize",
      format_xml_elements($quote_xml),
      [
        'Content-type: text/xml',
      ]
    );

    return $resource;
  }

  /**
   * Rejects a quote from Lionbridge.
   *
   * @param string $quote_id
   *   The quote id returned from Lionbridge.
   *
   * @return object
   *   Return xml with details for a single quote.
   *
   * @throws TMGMTException
   */
  public function rejectQuote($quote_id) {
    if (empty($quote_id)) {
      throw new TMGMTException('Empty Quote ID.');
    }

    return $this->sendResource('/api/quote/' . $quote_id . '/reject');
  }

  /**
   * Return xml with details for a single file.
   *
   * @param string $asset_id
   *   The asset id returned from Lionbridge.
   *
   * @return object
   *   Return xml with details for a single file.
   *
   * @throws TMGMTException
   */
  public function getFileDetails($asset_id) {
    if (empty($asset_id)) {
      throw new TMGMTException('Empty File asset ID.');
    }

    return $this->getResource('/api/files/' . $asset_id . '/details');
  }

  /**
   * Returns the content of a translated file.
   *
   * @param string $asset_id
   *   The asset id returned from Lionbridge.
   * @param string $lang_code
   *   The language code of the desired translation. Ex: 'fr-fr'.
   *
   * @return object
   *   The translated text.
   *
   * @throws TMGMTException
   */
  public function getFileTranslation($asset_id, $lang_code) {
    if (empty($asset_id)) {
      throw new TMGMTException('Empty File asset ID.');
    }
    if (empty($lang_code)) {
      throw new TMGMTException('Empty Language code.');
    }

    return $this->getResource('/api/files/' . $asset_id . '/' . $lang_code);
  }

  /**
   * Upload file for future translation.
   *
   * @param string $language_code
   *   The language code of the source text. Ex: 'fr-fr'.
   * @param string $file_name
   *   The name of the file.
   * @param string $content_type
   *   The content type of the file.
   * @param string $file_content
   *   The file contents.
   *
   * @return array
   *   The file information from Lionbridge, including the AssetID.
   */
  public function addFile($language_code, $file_name, $content_type, $file_content) {
    $resource = '/api/files/add/' . $language_code . '/' . $file_name;
    return $this->sendResource($resource, $file_content, [
      'Content-type:' . $content_type,
    ]);
  }

  /**
   * Creates a project from previously uploaded files.
   *
   * @param string $project_title
   *   The title of the project from the checkout settings form.
   * @param string $service_id
   *   The ID of the service chosen on the checkout settings form.
   * @param string $source_language
   *   The language code of the source data in the form of 'en-us'.
   * @param array $target_languages
   *   The language code of the desired translation in the form of 'fr-fr'.
   * @param array $products
   *   An array of product IDs for a product-based translation.
   * @param array $files
   *   An array of asset IDs for a file-based translation.
   *
   * @return mixed
   *   The project data from Lionbridge.
   */
  public function addProject($project_title, $service_id, $source_language, array $target_languages, array $products = [], array $files = []) {
    $project_xml = [
      'AddProject' => [
        'ProjectName' => $project_title,
        'TranslationOptions' => [
          'ServiceID' => $service_id,
          'SourceLanguage' => ['LanguageCode' => $source_language],
        ],
      ],
    ];

    foreach ($target_languages as $target_lang) {
      $project_xml['AddProject']['TranslationOptions']['TargetLanguages'][] = [
        'key' => 'TargetLanguage',
        'value' => ['LanguageCode' => $target_lang],
      ];
    }

    foreach ($files as $file) {
      $project_xml['AddProject']['Files'][] = [
        'key' => 'File',
        'value' => ['AssetID' => $file],
      ];
    }

    $project_xml['AddProject']['Products'] = '';
    foreach ($products as $product) {
      $project_xml['AddProject']['Products'] .= $product->toXml();
    }

    $result = $this->sendResource(
      '/api/projects/add',
      format_xml_elements($project_xml),
      [
        'Content-type: text/xml',
      ]
    );

    return $result;
  }

  /**
   * Generates a quote for uploaded files.
   *
   * @param string $project_id
   *   The  project id returned from addProject().
   * @param string $notification_url
   *   The URL to which Lionbirdge will POST a project complete notification.
   *
   * @return mixed
   *   The quote data from Lionbridge.
   */
  public function generateQuote($project_id, $notification_url) {
    $quote_xml = [
      'GenerateQuote' => [
        'TranslationOptions' => [
          'Currency' => $this->getCurrency(),
          'NotifyCompleteURL' => $notification_url,
        ],
        'Projects' => [
          [
            'key' => 'Project',
            'value' => ['ProjectID' => $project_id],
          ],
        ],
      ],
    ];

    $result = $this->sendResource(
      '/api/quote/generate',
      format_xml_elements($quote_xml),
      [
        'Content-type: text/xml',
      ]
    );
    return $result;
  }

  /**
   * Sends a GET request to Lionbridge services.
   *
   * @param string $resource
   *   The path to send the request.
   * @param string $method
   *   The request method.
   *
   * @return mixed
   *   Either text if it's a translation or and array with requested data.
   *
   * @throws TMGMTException
   */
  protected function getResource($resource, $method = 'GET') {
    $result = [];
    $options = [
      'headers' => $this->generateAuthHeaders($resource),
      'method' => $method,
    ];

    $url = $this->getEndPoint() . $resource;

    try {
      $response = drupal_http_request($url, $options);

      if ($response->code != 200) {
        $resource_components = explode('/', $resource);
        $resource_type = rtrim($resource_components[2], 's');
        throw new TMGMTException('Issue with ' . $resource_type . ' resource - ' . $response->error);
      }
    }
    catch (TMGMTException $e) {
      watchdog_exception('lionbridge_translation_provider', $e);
      drupal_set_message($e->getMessage(), 'error');
    }

    $content_type = isset($response->headers['content-type']) ? $response->headers['content-type'] : NULL;
    switch ($content_type) {
      case 'text/plain':
      case 'application/json':
        $result = $response->data;
        break;

      default:
        $xml_data = isset($response->data) ? $response->data : NULL;
        @$xml_obj = simplexml_load_string($xml_data);
        if (!$xml_obj) {
          $result = [
            'Error' => 'Invalid Lionbridge onDemand endpoint.',
          ];
        }
        else {
          $result_data = $this->xmlToArray($xml_obj);
          if ($result_data) {
            // Use received error message.
            if (isset($result_data['Error'])) {
              $result = [
                'Error' => $result_data['Error'],
              ];
            }
            else {
              $result = $result_data;
            }
          }
        }
        break;
    }

    return $result;
  }

  /**
   * Sends a POST request to Lionbridge.
   *
   * @param string $resource
   *   The path to send the request.
   * @param string $data
   *   Data to be sent to Lionbridge.
   * @param array $additional_headers
   *   Any additional request headers to be sent.
   *
   * @return mixed
   *   The XML data returned from Lionbridge converted to an array.
   */
  protected function sendResource($resource, $data = '', array $additional_headers = []) {
    $auth_headers = $this->generateAuthHeaders($resource, 'POST');

    // Concatenate keys and values of auth array.
    foreach ($auth_headers as $key => $value) {
      $headers[] = $key . ': ' . $value;
    }

    if (!empty($headers)) {
      $request_headers = array_merge($headers, $additional_headers);
    }

    $url = $this->getEndPoint() . $resource;

    $url = url($url,
      [
        'absolute' => TRUE,
        'external' => TRUE,
        'https' => TRUE,
      ]
    );

    $request = curl_init();
    curl_setopt($request, CURLOPT_URL, $url);
    curl_setopt($request, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($request, CURLOPT_POST, 1);
    curl_setopt($request, CURLOPT_POSTFIELDS, $data);
    curl_setopt($request, CURLOPT_HTTPHEADER, $request_headers);
    $response = curl_exec($request);
    curl_close($request);

    @$xml_obj = simplexml_load_string($response);
    $result_data = $this->xmlToArray($xml_obj);

    return $result_data;
  }

  /**
   * Converts XML to an array.
   *
   * @param \SimpleXMLElement $xml_object
   *   The XML returned from Lionbridge.
   *
   * @return mixed
   *   An array of data converted from the XML object.
   */
  protected function xmlToArray(\SimpleXMLElement $xml_object) {
    return json_decode(json_encode((array) $xml_object), TRUE);
  }

  /**
   * Generates the authorization header required by Lionbridge.
   *
   * @param string $resource
   *   The path to which the request will be sent.
   *
   * @return array
   *   An array of authorization headers.
   */
  public function generateAuthHeaders($resource, $method = 'GET', $micro_time = NULL) {
    date_default_timezone_set('America/New_York');
    if (is_null($micro_time)) {
      $micro_time = microtime(TRUE);
    }
    $micro = sprintf("%06d", ($micro_time - floor($micro_time)) * 1000000);
    $d = new \DateTime(date('Y-m-d H:i:s.' . $micro, $micro_time));
    $date_time = $d->format('Y-m-d\TH:i:s.u');

    $signature = "{$method}:{$resource}:{$this->getSecretId()}:{$date_time}:{$this->getApiVersion()}:text/xml";
    $signature = base64_encode(hash('sha256', $signature, TRUE));
    $authorization = "LOD1-BASE64-SHA256 KeyID={$this->getKeyId()},Signature={$signature},SignedHeaders=x-lod-timestamp;x-lod-version;accept";

    return [
      "Accept" => "text/xml",
      "Authorization" => $authorization,
      "x-lod-version" => $this->getApiVersion(),
      "x-lod-timestamp" => $date_time,
    ];
  }

  /**
   * Count all content types supporting multilingual support.
   */
  public function getMultilingualSupportCtCount() {
    $cts = node_type_get_types();
    $i = 0;
    foreach ($cts as $ct) {
      if (variable_get('language_content_type_' . $ct->type) > 0) {
        $i++;
      }
    }

    return $i;
  }

}
