<?php

/**
 * @file
 * Controller class for the Lionbridge translator ui.
 */

/**
 * Lionbridge translator ui controller.
 */
class TMGMTLionbridgeTranslatorUIController extends TMGMTDefaultTranslatorUIController {

  /**
   * Overrides TMGMTDefaultTranslatorUIController::pluginSettingsForm().
   */
  public function pluginSettingsForm($form, &$form_state, TMGMTTranslator $translator, $busy = FALSE) {
    $api_client = new TMGMTLionbridgeConnector($translator);

    // Check if there is more than one language enabled.
    if (count(language_list()) === 1) {
      drupal_set_message(t('Only one language has been detected, please <a href="@url_add">add new language</a> here.  To see list of languages, <a href="@url_list" target="_blank">click here</a>.', [
        '@url_add' => url('admin/config/regional/language/add&destination=' . current_path()),
        '@url_list' => url('admin/config/regional/language'),
      ]), 'warning');
    }

    // Check if there is at least one CT with multilingual support enabled.
    if ($api_client->getMultilingualSupportCtCount() === 0) {
      drupal_set_message(t('There must be at least one content type with a multilingual support enabled.'), 'warning');
    }

    $form['endpoint'] = [
      '#type' => 'textfield',
      '#title' => t('Liondemand Endpoint'),
      '#default_value' => $translator->getSetting('endpoint'),
      '#description' => t('Please enter your Lionbridge liondemand endpoint url.'),
      '#element_validate' => ['_lionbridge_translation_provider_validate_endpoint_url'],
      '#required' => TRUE,
    ];

    $form['access_key_id'] = [
      '#type' => 'textfield',
      '#title' => t('Lionbridge Access Key Id'),
      '#default_value' => $translator->getSetting('access_key_id'),
      '#description' => t('Please enter your Lionbridge Access Key ID.'),
      '#required' => TRUE,
      '#maxlength' => 20,
    ];

    $form['access_key'] = [
      '#type' => 'textfield',
      '#title' => t('Lionbridge Access Key'),
      '#default_value' => $translator->getSetting('access_key'),
      '#description' => t('Please enter your Lionbridge Access Key.'),
      '#required' => TRUE,
      '#maxlength' => 40,
    ];

    $form['po_number'] = [
      '#type' => 'textfield',
      '#title' => t('Purchase Order Number'),
      '#required' => TRUE,
      '#default_value' => 123456,
      '#description' => t('Please enter your Lionbridge Purchase Order Number.'),
    ];

    // Only show service, currency and account info if translator is available.
    if ($translator->isAvailable() == TRUE) {
      $account_info = $api_client->accountInformation();
      $services = $api_client->listServices();
      $currency = $api_client->getAllowedCurrencies();

      $no_errors = 1;
      if (isset($account_info['head']) || isset($account_info['body'])) {
        $no_errors = 0;
      }
      elseif (isset($account_info['Error'])) {
        $no_errors = 0;
      }

      if ($no_errors == 1) {
        $form['po_number']['#default_value'] = $translator->getSetting('po_number');

        if (is_array($services)) {
          $options = [];

          foreach ($services['Service'] as $service) {
            $options[$service['ServiceID']] = $service['Name'];
          }

          $form['job_type'] = [
            '#type' => 'select',
            '#title' => t('Job Type'),
            '#options' => [
              'translation' => t('Translation'),
              'tm_update' => t('Translation Memory Update'),
            ],
            '#default_value' => $translator->getSetting('job_type'),
          ];

          $form['service'] = [
            '#type' => 'select',
            '#title' => t('Default translation service'),
            '#options' => $options,
            '#default_value' => $translator->getSetting('service'),
          ];
        }

        if (is_array($currency)) {
          $form['currency'] = [
            '#type' => 'select',
            '#title' => t('Default currency'),
            '#options' => drupal_map_assoc($currency),
            '#default_value' => $translator->getSetting('currency'),
          ];
        }

        $form['account_info'] = [
          '#type' => 'fieldset',
          '#title' => t('Lionbridge Account Info'),
          '#collapsible' => TRUE,
          '#collapsed' => TRUE,
        ];

        if (is_array($account_info)) {
          unset($account_info['TargetLanguages']);
          foreach ($account_info as $key => $val) {
            if (is_array($val)) {
              $val = implode('<br>', $val);
            }
            $form['account_info'][$key] = [
              '#markup' => '<strong>' . $key . ':</strong> ' . $val . '<br>',
            ];
          }
        }
      }
    }

    return parent::pluginSettingsForm($form, $form_state, $translator);
  }

  /**
   * Overrides TMGMTDefaultTranslatorUIController::checkoutSettingsForm().
   */
  public function checkoutSettingsForm($form, &$form_state, TMGMTJob $job) {
    $settings['project_title'] = [
      '#type' => 'textfield',
      '#title' => t('Project title'),
      '#size' => 60,
      '#default_value' => $job->label(),
    ];

    $settings['project_key'] = [
      '#type' => 'value',
      '#value' => !empty($job->settings['project_key']) ? $job->settings['project_key'] : drupal_random_key(),
    ];

    return $settings;
  }

  /**
   * Implements TMGMTTranslatorUIControllerInterface::checkoutInfo().
   */
  public function checkoutInfo(TMGMTJob $job) {
    $form = [];
    $translator = $job->getTranslator();
    $api_client = new TMGMTLionbridgeConnector($translator);
    $quote = $api_client->getQuote($job->reference);

    if (is_array($quote) && isset($quote['Error'])) {
      return [
        ['error' => ['#markup' => t('Quote is not available.')]],
      ];
    }

    if (isset($quote['QuoteID'])) {
      $form['quote'] = [
        '#type' => 'item',
        '#title' => t('Quote: @quote_id', ['@quote_id' => $quote['QuoteID']]),
      ];
    }

    if (isset($quote['TotalCost']) && isset($quote['Currency'])) {
      // Account for empty values, which would set cost to 0.
      $cost = empty($quote['TotalCost']) ? 0 : $quote['TotalCost'];

      $form['quote']['cost'] = [
        '#type' => 'item',
        '#markup' => t('<strong>Total Cost:</strong> @cost @currency', [
          '@cost' => $cost,
          '@currency' => $quote['Currency'],
        ]),
      ];
    }

    if (isset($quote['Status'])) {
      $quote_status = $quote['Status'];

      if ($job->isFinished() && $quote['Status'] === LIONBRIDGE_TRANSLATION_PROVIDER_QUOTE_STATUS_AUTHORIZED) {
        $quote_status .= ' - Translation is pending revision from the provider';
      }

      $form['quote']['status'] = [
        '#type' => 'item',
        '#markup' => t('<strong>Status:</strong> @status', [
          '@status' => $quote_status,
        ]),
      ];

      if ($quote['Status'] == LIONBRIDGE_TRANSLATION_PROVIDER_QUOTE_STATUS_PENDING) {
        $form['actions']['authorize'] = [
          '#type' => 'submit',
          '#value' => t('Authorize Quote'),
          '#submit' => ['_lionbridge_translation_provider_authorize_quote'],
          '#weight' => -10,
        ];
      }
    }

    if ($job->isActive()) {
      $form['actions']['poll'] = [
        '#type' => 'submit',
        '#value' => t('Poll translations'),
        '#submit' => ['_lionbridge_translation_provider_poll_submit'],
        '#weight' => -10,
      ];
    }

    if ($job->isFinished()
      && (isset($quote['Status']) && $quote['Status'] == LIONBRIDGE_TRANSLATION_PROVIDER_QUOTE_STATUS_COMPLETE)
      && (isset($translator->settings['job_type']) && $translator->settings['job_type'] != 'tm_update')) {
      $form['actions']['redeliver'] = [
        '#type' => 'submit',
        '#value' => t('Force re-download'),
        '#submit' => ['_lionbridge_translation_provider_redeliver'],
        '#weight' => -10,
        '#suffix' => t('WARNING: Re-downloaded translations may not contain edits from the previous review.'),
      ];
    }

    return $form;
  }

}
